@@ -55,7 +55,8 @@
 
 #include <e-util/e-account.h>
 #include <e-util/e-account-list.h>
-#include <e-util/e-passwords.h>
+#include <libedataserverui/e-passwords.h>
+#include <e-util/e-dialog-utils.h>
 #include <gconf/gconf-client.h>
 #include <libxml/tree.h>
 #include <libxml/HTMLparser.h>
@@ -1388,41 +1389,147 @@ e2k_autoconfig_lookup_option (const char *option)
 }
 
 static gboolean 
-validate (char *owa_url, char *user, char *password, char **host)
+validate (const char *owa_url, char *user, char *password, ExchangeParams *exchange_params)
 {
 	E2kAutoconfig *ac;
 	E2kOperation op;        /* FIXME */
 	E2kAutoconfigResult result;
+	E2kUri *euri;
 	gboolean valid = FALSE;
+	const char *old, *new, *path;
 
 	ac = e2k_autoconfig_new (owa_url, user, password, 
 				 E2K_AUTOCONFIG_USE_EITHER);
 
 	e2k_operation_init (&op);
-	//e2k_autoconfig_set_gc_server (ac, ad_server, gal_limit) FIXME
-	e2k_autoconfig_set_gc_server (ac, NULL, -1);
+	// e2k_autoconfig_set_gc_server (ac, ad_server, gal_limit) FIXME
+	// e2k_autoconfig_set_gc_server (ac, NULL, -1);
 	result = e2k_autoconfig_check_exchange (ac, &op);
 
 	if (result == E2K_AUTOCONFIG_OK) {
 		result = e2k_autoconfig_check_global_catalog (ac, &op);
 		e2k_operation_free (&op);
-		/* Need to fill GC server and return */
+		
+		/* find mailbox and owa_path values */	
+		euri = e2k_uri_new (ac->home_uri);
+		exchange_params->owa_path = g_strdup (euri->path + 1);
+		e2k_uri_free (euri);
+		path = exchange_params->owa_path;
+		exchange_params->mailbox = strrchr (path, '/');
+		if (exchange_params->mailbox && !exchange_params->mailbox[1]) {
+			*exchange_params->mailbox = '\0';
+			exchange_params->mailbox = strrchr (path, '/');
+		}
+		if (exchange_params->mailbox)
+			*exchange_params->mailbox++ = '\0';
+
+		exchange_params->host = ac->pf_server;
+		if (ac->gc_server) 
+			exchange_params->ad_server = ac->gc_server;
+
 		valid = TRUE;
 	}
 	else {
-#if 0
-		e2k_operation_free (&op);
-		window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-		valid = e2k_autoconfig_handle_error (result, window);
-#endif
-		valid = FALSE; /* FIXME return valid */
+		switch (result) {
+
+		case E2K_AUTOCONFIG_CANT_CONNECT:
+			if (!strncmp (ac->owa_uri, "http:", 5)) {
+				old = "http";
+				new = "https";
+			} else {
+				old = "https";
+				new = "http";
+			}
+
+			e_notice (NULL, GTK_MESSAGE_ERROR,
+				  _("Could not connect to the Exchange "
+			    	    "server.\nMake sure the URL is correct "
+			    	    "(try \"%s\" instead of \"%s\"?) "
+			    	    "and try again."), new, old);
+			valid = FALSE;
+			break;
+
+		case E2K_AUTOCONFIG_CANT_RESOLVE:
+			e_notice (NULL, GTK_MESSAGE_ERROR,
+			  	_("Could not locate Exchange server.\n"
+			    	  "Make sure the server name is spelled correctly "
+			    	  "and try again."));
+			valid = FALSE;
+			break;
+
+		case E2K_AUTOCONFIG_AUTH_ERROR:
+		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_NTLM:
+		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_BASIC:
+			e_notice (NULL, GTK_MESSAGE_ERROR,
+			  	_("Could not authenticate to the Exchange "
+			    	  "server.\nMake sure the username and "
+			    	  "password are correct and try again."));
+			valid = FALSE;
+			break;
+
+		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_DOMAIN:
+			e_notice (NULL, GTK_MESSAGE_ERROR,
+			  	_("Could not authenticate to the Exchange "
+			    	  "server.\nMake sure the username and "
+			    	  "password are correct and try again.\n\n"
+			    	  "You may need to specify the Windows "
+			    	  "domain name as part of your username "
+			    	  "(eg, \"MY-DOMAIN\\%s\")."),
+			  	  ac->username);
+			valid = FALSE;
+			break;
+
+		case E2K_AUTOCONFIG_NO_OWA:
+		case E2K_AUTOCONFIG_NOT_EXCHANGE:
+			e_notice (NULL, GTK_MESSAGE_ERROR,
+			  	_("Could not find OWA data at the indicated URL.\n"
+			    	  "Make sure the URL is correct and try again."));
+			valid = FALSE;
+			break;
+
+		case E2K_AUTOCONFIG_CANT_BPROPFIND:
+			e_notice (
+				NULL, GTK_MESSAGE_ERROR,
+				_("Ximian Connector requires access to certain "
+			  	"functionality on the Exchange Server that appears "
+			  	"to be disabled or blocked.  (This is usually "
+			  	"unintentional.)  Your Exchange Administrator will "
+			  	"need to enable this functionality in order for "
+			  	"you to be able to use Ximian Connector.\n\n"
+			  	"For information to provide to your Exchange "
+			  	"administrator, please follow the link below:\n"
+				"http://support.novell.com/cgi-bin/search/searchtid.cgi?/ximian/ximian328.html "));
+
+			valid = FALSE;
+			break;
+
+		case E2K_AUTOCONFIG_EXCHANGE_5_5:
+			e_notice (
+				NULL, GTK_MESSAGE_ERROR,
+				_("The Exchange server URL you provided is for an "
+			  	"Exchange 5.5 Server. Ximian Connector supports "
+			  	"Microsoft Exchange 2000 and 2003 only."));
+
+			valid = FALSE;
+			break;
+
+		default:
+			e_notice (NULL, GTK_MESSAGE_ERROR,
+			  	_("Could not configure Exchange account because "
+			    	  "an unknown error occurred. Check the URL, "
+			    	  "username, and password, and try again."));
+			valid = FALSE; /* FIXME return valid */
+			break;
+		}
 	}
-	*host = g_strdup (ac->pf_server);
+
+
 	return valid;
 }
 
 gboolean
-e2k_validate_user (char *owa_url, char *user, char **host)
+e2k_validate_user (const char *owa_url, char *user,
+		   ExchangeParams *exchange_params, gboolean *remember_password)
 {
 	gboolean valid = FALSE, remember=FALSE;
 	char *key, *password, *prompt;
@@ -1436,11 +1543,28 @@ e2k_validate_user (char *owa_url, char *user, char **host)
 					E_PASSWORDS_REMEMBER_FOREVER|E_PASSWORDS_SECRET,
 					&remember, NULL);
 		if (password) {
-			valid = validate (owa_url, user, password, host);
-			//auto_detect_gc();
+			valid = validate (owa_url, user, password, exchange_params);
+			if (valid) {
+				/* generate the proper key once the host name 
+				 * is read and remember password temporarily, 
+				 * so that at the end of * account creation, 
+				 * user will not be prompted, for password will
+				 * not be asked again. 
+				 */
+
+				*remember_password = remember;
+				g_free (key);
+				key = g_strdup_printf ("%s//%s@%s", 
+						       "exchange:", user, exchange_params->host);
+				e_passwords_add_password (key, password);
+				e_passwords_remember_password ("Exchange", key);
+			}
 		}
 		g_free (prompt);
 	}
+	if (password && !valid) {
+		e_passwords_forget_password ("Exchange", key);
+	}
 	g_free (key);
 
 	return valid;
