@@ -32,7 +32,6 @@
 #include "exchange-hierarchy-favorites.h"
 #include "exchange-hierarchy-foreign.h"
 #include "exchange-hierarchy-gal.h"
-#include "exchange-offline-listener.h"
 #include "e-folder-exchange.h"
 #include "e2k-autoconfig.h"
 #include "e2k-encoding-utils.h"
@@ -42,7 +41,7 @@
 #include "e2k-uri.h"
 #include "e2k-utils.h"
 #include <e-util/e-dialog-utils.h>
-#include <e-util/e-passwords.h>
+#include <libedataserverui/e-passwords.h>
 
 #include <gal/util/e-util.h>
 #include <libgnome/gnome-util.h>
@@ -76,12 +75,13 @@ struct _ExchangeAccountPrivate {
 	ExchangeHierarchy *favorites_hierarchy;
 	GHashTable *folders;
 	char *uri_authority, *http_uri_schema;
-	gboolean uris_use_email;
+	gboolean uris_use_email, offline_sync;
 
 	char *identity_name, *identity_email, *source_uri, *password_key;
-	char *username, *password, *windows_domain, *ad_server;
+	char *username, *password, *windows_domain, *nt_domain, *ad_server;
+	char *owa_url;
 	E2kAutoconfigAuthPref auth_pref;
-	int ad_limit;
+	int ad_limit, passwd_exp_warn_period;
 
 	EAccountList *account_list;
 	EAccount *account;
@@ -158,6 +158,7 @@ init (GObject *object)
 	account->priv->folders = g_hash_table_new (g_str_hash, g_str_equal);
 	account->priv->discover_data_lock = g_mutex_new ();
 	account->priv->account_online = TRUE;
+	account->priv->nt_domain = NULL;
 }
 
 static void
@@ -282,9 +283,15 @@ finalize (GObject *object)
 	if (account->priv->windows_domain)
 		g_free (account->priv->windows_domain);
 
+	if (account->priv->nt_domain)
+		g_free (account->priv->nt_domain);
+
 	if (account->priv->ad_server)
 		g_free (account->priv->ad_server);
 
+	if (account->priv->owa_url)
+		g_free (account->priv->owa_url);
+
 	if (account->priv->connect_lock)
 		g_mutex_free (account->priv->connect_lock);
 
@@ -706,6 +713,7 @@ exchange_account_open_folder (ExchangeAccount *account, const char *path)
 {
 	ExchangeHierarchy *hier;
 	EFolder *folder;
+	int offline;
 
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), 
 				EXCHANGE_ACCOUNT_FOLDER_GENERIC_ERROR);
@@ -713,7 +721,8 @@ exchange_account_open_folder (ExchangeAccount *account, const char *path)
 	if (!get_folder (account, path, &folder, &hier))
 		return EXCHANGE_ACCOUNT_FOLDER_DOES_NOT_EXIST;
 
-	if (!account->priv->connected &&
+	exchange_account_is_offline (account, &offline);
+	if (offline == ONLINE_MODE && !account->priv->connected &&
 	    hier == (ExchangeHierarchy *)account->priv->hierarchies->pdata[0] &&
 	    folder == hier->toplevel) {
 		/* The shell is asking us to open the personal folders
@@ -722,8 +731,8 @@ exchange_account_open_folder (ExchangeAccount *account, const char *path)
 		 */
 		return EXCHANGE_ACCOUNT_FOLDER_DOES_NOT_EXIST;
 	}		
-
-	return exchange_hierarchy_scan_subtree (hier, folder);
+	
+	return exchange_hierarchy_scan_subtree (hier, folder, (offline == OFFLINE_MODE));
 }
 
 ExchangeAccountFolderResult
@@ -840,21 +849,35 @@ get_password (ExchangeAccount *account, E2kAutoconfig *ac, const char *errmsg)
 		e_passwords_forget_password ("Exchange", account->priv->password_key);
 
 	password = e_passwords_get_password ("Exchange", account->priv->password_key);
-	if (!password && exchange_component_is_interactive (global_exchange_component)) {
-		char *prompt;
 
-		prompt = g_strdup_printf (_("%sEnter password for %s"),
-					  errmsg, account->account_name);
-		oldremember = remember = account->priv->account->source->save_passwd;
-		password = e_passwords_ask_password (
-			_("Enter password"),
-			"Exchange", account->priv->password_key,
-			prompt, E_PASSWORDS_REMEMBER_FOREVER|E_PASSWORDS_SECRET,
-			&remember, NULL);
-		if (remember != oldremember) {
-			account->priv->account->source->save_passwd = remember;
+	if (exchange_component_is_interactive (global_exchange_component)) {
+		if (!password) {
+			char *prompt;
+
+			prompt = g_strdup_printf (_("%sEnter password for %s"),
+						  errmsg, account->account_name);
+			oldremember = remember = 
+					account->priv->account->source->save_passwd;
+			password = e_passwords_ask_password (
+					_("Enter password"),
+					"Exchange", 
+					account->priv->password_key,
+					prompt, 
+					E_PASSWORDS_REMEMBER_FOREVER|E_PASSWORDS_SECRET,
+					&remember, 
+					NULL);
+			if (remember != oldremember) {
+				account->priv->account->source->save_passwd = remember;
+			}
+			g_free (prompt);
+		} 
+		else if (!account->priv->account->source->save_passwd) {
+			/* get_password returns the password cached but user has not 
+		 	 * selected remember password option, forget this password 
+		 	 * whis is stored temporarily by e2k_validate_user() 
+		 	 */
+			e_passwords_forget_password ("Exchange", account->priv->password_key);
 		}
-		g_free (prompt);
 	}
 
 	if (password) {
@@ -889,19 +912,34 @@ is_password_expired (ExchangeAccount *account, E2kAutoconfig *ac)
 
 	result = e2k_kerberos_check_password (ac->username, domain,
 					      ac->password);
+	if (result != E2K_KERBEROS_OK || 
+	    result != E2K_KERBEROS_PASSWORD_EXPIRED) {
+		/* try again with nt domain */
+		domain = ac->nt_domain;
+		if (domain)
+			result = e2k_kerberos_check_password (ac->username, 
+							      domain,
+							      ac->password);
+	} 
 
 	return (result == E2K_KERBEROS_PASSWORD_EXPIRED);
 }
 
 static void
-change_passwd_cb (GtkWidget *button, char *old_passwd)
+change_passwd_cb (GtkWidget *button, ExchangeAccount *account)
 {
+	char *current_passwd, *new_passwd;
+
 	gtk_widget_hide (gtk_widget_get_toplevel(button));
-	exchange_get_new_password (old_passwd, TRUE);
+	current_passwd = exchange_account_get_password (account);
+	new_passwd = exchange_get_new_password (current_passwd, TRUE);
+	exchange_account_set_password (account, current_passwd, new_passwd);
+	g_free (current_passwd);
+	g_free (new_passwd);
 }
 
 static void
-display_passwd_expiry_message (int max_passwd_age, char *old_passwd)
+display_passwd_expiry_message (int max_passwd_age, ExchangeAccount *account)
 {
 	GladeXML *xml;
 	GtkWidget *top_widget, *change_passwd_button;
@@ -926,7 +964,7 @@ display_passwd_expiry_message (int max_passwd_age, char *old_passwd)
 	g_signal_connect (change_passwd_button, 
 			  "clicked", 
 			  G_CALLBACK (change_passwd_cb), 
-			  old_passwd);
+			  account);
 
 	response = gtk_dialog_run (GTK_DIALOG (top_widget));
 
@@ -942,7 +980,10 @@ find_passwd_exp_period (ExchangeAccount *account, E2kGlobalCatalogEntry *entry)
 	int max_pwd_age_days;
 	E2kOperation gcop;
 	E2kGlobalCatalogStatus gcstatus;
-	char *current_passwd;
+
+	/* If user has not selected password expiry warning option, return */
+	if (account->priv->passwd_exp_warn_period == -1)
+		return;
 
 	/* Check for password expiry period */ 
 	/* This needs to be invoked after is_password_expired(), i.e., 
@@ -976,11 +1017,9 @@ find_passwd_exp_period (ExchangeAccount *account, E2kGlobalCatalogEntry *entry)
 		max_pwd_age_days = 
 		( max_pwd_age * ONE_HUNDRED_NANOSECOND ) / SECONDS_IN_DAY;
 
-		if (max_pwd_age_days <= PASSWD_EXPIRY_NOTIFICATION_PERIOD) {
-			/* This we need for changing password */
-			current_passwd = exchange_account_get_password (account);
+		if (max_pwd_age_days <= account->priv->passwd_exp_warn_period) {
 			display_passwd_expiry_message (max_pwd_age_days, 
-						       current_passwd);
+						       account);
 		}
 	} 
 }
@@ -1014,12 +1053,21 @@ exchange_account_set_password (ExchangeAccount *account, char *old_pass, char *n
 			domain++;
 	}
 	if (!domain) {
+		/* email id is not proper, we return instead of trying nt_domain */
 		e_notice (NULL, GTK_MESSAGE_ERROR, _("Cannot change password due to configuration problems"));
 		return;
 	}
 
 	result = e2k_kerberos_change_password (account->priv->username, domain,
 					       old_pass, new_pass);
+	if (result != E2K_KERBEROS_OK || result != E2K_KERBEROS_PASSWORD_TOO_WEAK) {
+		/* try with nt_domain */
+		domain = account->priv->nt_domain;
+		if (domain)
+			result = e2k_kerberos_change_password (account->priv->username, 
+							       domain, old_pass,
+							       new_pass);
+	}
 	switch (result) {
 	case E2K_KERBEROS_OK:
 		e_passwords_forget_password ("Exchange", account->priv->password_key);
@@ -1039,7 +1087,6 @@ exchange_account_set_password (ExchangeAccount *account, char *old_pass, char *n
 	}
 }
 
-#ifdef OFFLINE_SUPPORTED
 /**
  * exchange_account_set_offline:
  * @account: an #ExchangeAccount
@@ -1080,18 +1127,154 @@ exchange_account_set_offline (ExchangeAccount *account)
 gboolean
 exchange_account_set_online (ExchangeAccount *account)
 {
-		
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), FALSE);
 
-	account->priv->account_online = TRUE;
+	if (!account->priv->account_online) {
+		account->priv->account_online = TRUE;
 
-	if (exchange_account_connect (account))
+		if (exchange_account_connect (account))
+			return TRUE;
+		else
+			return FALSE;
+	} else {
 		return TRUE;
-	else
+	}
+}
+
+/**
+ * exchange_account_is_offline:
+ * @account: an #ExchangeAccount
+ *
+ * Return value: Returns TRUE if account is offline
+ **/
+void
+exchange_account_is_offline (ExchangeAccount *account, int *state)
+{
+	*state = UNSUPPORTED_MODE;
+
+	g_return_if_fail (EXCHANGE_IS_ACCOUNT (account));
+
+	exchange_component_is_offline (global_exchange_component, state);
+}
+
+
+static gboolean
+setup_account_hierarchies (ExchangeAccount *account)
+{
+	ExchangeHierarchy *hier, *personal_hier;
+	ExchangeAccountFolderResult fresult;
+	char *phys_uri_prefix, *dir;
+	DIR *d;
+	struct dirent *dent;
+	int offline;
+
+	exchange_account_is_offline (account, &offline);
+
+	if (offline == UNSUPPORTED_MODE)
 		return FALSE;
+
+	/* Set up Personal Folders hierarchy */
+	phys_uri_prefix = g_strdup_printf ("exchange://%s/personal",
+					   account->priv->uri_authority);
+	hier = exchange_hierarchy_webdav_new (account,
+					      EXCHANGE_HIERARCHY_PERSONAL,
+					      _("Personal Folders"),
+					      phys_uri_prefix,
+					      account->home_uri,
+					      account->priv->identity_name,
+					      account->priv->identity_email,
+					      account->priv->source_uri,
+					      TRUE);
+	setup_hierarchy (account, hier);
+	g_free (phys_uri_prefix);
+	personal_hier = hier;
+
+	/* Favorite Public Folders */
+	phys_uri_prefix = g_strdup_printf ("exchange://%s/favorites",
+					   account->priv->uri_authority);
+	hier = exchange_hierarchy_favorites_new (account,
+						 _("Favorite Public Folders"),
+						 phys_uri_prefix,
+						 account->home_uri,
+						 account->public_uri,
+						 account->priv->identity_name,
+						 account->priv->identity_email,
+						 account->priv->source_uri);
+	setup_hierarchy (account, hier);
+	g_free (phys_uri_prefix);
+	account->priv->favorites_hierarchy = hier;
+
+	/* Public Folders */
+	phys_uri_prefix = g_strdup_printf ("exchange://%s/public",
+					   account->priv->uri_authority);
+	hier = exchange_hierarchy_webdav_new (account,
+					      EXCHANGE_HIERARCHY_PUBLIC,
+					      /* i18n: Outlookism */
+					      _("All Public Folders"),
+					      phys_uri_prefix,
+					      account->public_uri,
+					      account->priv->identity_name,
+					      account->priv->identity_email,
+					      account->priv->source_uri,
+					      FALSE);
+	setup_hierarchy (account, hier);
+	g_free (phys_uri_prefix);
+
+	/* Global Address List */
+	phys_uri_prefix = g_strdup_printf ("gal://%s/gal",
+					   account->priv->uri_authority);
+						     /* i18n: Outlookism */
+	hier = exchange_hierarchy_gal_new (account, _("Global Address List"),
+					   phys_uri_prefix);
+	setup_hierarchy (account, hier);
+	g_free (phys_uri_prefix);
+
+	/* Other users' folders */
+	d = opendir (account->storage_dir);
+	if (d) {
+		while ((dent = readdir (d))) {
+			if (!strchr (dent->d_name, '@'))
+				continue;
+			dir = g_strdup_printf ("%s/%s", account->storage_dir,
+					       dent->d_name);
+			hier = exchange_hierarchy_foreign_new_from_dir (account, dir);
+			g_free (dir);
+			if (!hier)
+				continue;
+
+			setup_hierarchy_foreign (account, hier);
+		}
+		closedir (d);
+	}
+
+	/* Scan the personal and favorite folders so we can resolve references
+	 * to the Calendar, Contacts, etc even if the tree isn't
+	 * opened.
+	 */
+	fresult = exchange_hierarchy_scan_subtree (personal_hier,
+						   personal_hier->toplevel,
+						   (offline == OFFLINE_MODE));
+	if (fresult != EXCHANGE_ACCOUNT_FOLDER_OK) {
+		account->priv->connecting = FALSE;
+		return FALSE;
+	}
+
+	account->mbox_size = exchange_hierarchy_webdav_get_total_folder_size (
+					EXCHANGE_HIERARCHY_WEBDAV (personal_hier));
+
+	fresult = exchange_hierarchy_scan_subtree (
+		account->priv->favorites_hierarchy,
+		account->priv->favorites_hierarchy->toplevel,
+		(offline == OFFLINE_MODE));
+	if (fresult != EXCHANGE_ACCOUNT_FOLDER_OK && 
+	    fresult != EXCHANGE_ACCOUNT_FOLDER_DOES_NOT_EXIST) {
+		account->priv->connecting = FALSE;
+		return FALSE;
+	}
+	
+	return TRUE;
 }
 
-#endif
 /**
  * exchange_account_connect:
  * @account: an #ExchangeAccount
@@ -1108,7 +1291,6 @@ exchange_account_connect (ExchangeAccount *account)
 {
 	E2kAutoconfig *ac;
 	E2kAutoconfigResult result;
-	ExchangeAccountFolderResult fresult;
 	E2kHTTPStatus status;
 	char *errmsg = "";
 	gboolean redirected = FALSE;
@@ -1116,26 +1298,25 @@ exchange_account_connect (ExchangeAccount *account)
 	int nresults;
 	GByteArray *entryid;
 	const char *timezone;
-	char *phys_uri_prefix, *dir;
-	ExchangeHierarchy *hier, *personal_hier;
-	struct dirent *dent;
-	DIR *d;
 	char *old_password, *new_password;
 	E2kGlobalCatalogStatus gcstatus;
 	E2kGlobalCatalogEntry *entry;
 	E2kOperation gcop;
 	const char *quota_msg = NULL;
 	char *user_name = NULL;
+	int offline;
 
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
 
+	exchange_account_is_offline (account, &offline);
 	g_mutex_lock (account->priv->connect_lock);
-#ifdef OFFLINE_SUPPORTED
-	if ((account->priv->connecting) || (!account->priv->account_online)){
-#else
-	if (account->priv->connecting) {
-#endif
+
+	if ((account->priv->connecting) || (offline == OFFLINE_MODE)){
 		g_mutex_unlock (account->priv->connect_lock);
+		if (offline == OFFLINE_MODE) {
+			setup_account_hierarchies (account);
+			e_notice (NULL, GTK_MESSAGE_ERROR, _("Exchange Account is offline. Cannot display folders\n"));
+		}
 		return NULL;
 	} else if (account->priv->ctx) {
 		g_mutex_unlock (account->priv->connect_lock);
@@ -1168,6 +1349,11 @@ exchange_account_connect (ExchangeAccount *account)
  try_connect_again:
 	account->priv->ctx = e2k_autoconfig_get_context (ac, NULL, &result);
 
+	if (!account->priv->nt_domain && ac->nt_domain)
+		account->priv->nt_domain = g_strdup (ac->nt_domain);
+	else
+		account->priv->nt_domain = NULL;
+
 	if (result != E2K_AUTOCONFIG_OK) {
 		if ( is_password_expired (account, ac)) {
 			old_password = exchange_account_get_password (account);
@@ -1187,7 +1373,6 @@ exchange_account_connect (ExchangeAccount *account)
 
 		switch (result) {
 		case E2K_AUTOCONFIG_AUTH_ERROR:
-		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_NTLM:
 			errmsg = _("Could not authenticate to server. "
 				   "(Password incorrect?)\n\n");
 			goto try_password_again;
@@ -1203,13 +1388,13 @@ exchange_account_connect (ExchangeAccount *account)
 				   "password wrong.\n\n");
 			goto try_password_again;
 
+		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_NTLM:
+			ac->use_ntlm = 1;
+			goto try_connect_again;
+
 		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_BASIC:
-			errmsg = _("Could not authenticate to server."
-				   "\n\nYou may need to use Plaintext "
-				   "Password authentication.\n\nOr "
-				   "you might have just typed your "
-				   "password wrong.\n\n");
-			goto try_password_again;
+			ac->use_ntlm = 0;
+			goto try_connect_again;
 
 		case E2K_AUTOCONFIG_REDIRECT:
 			if (!redirected && account_moved (account, ac))
@@ -1316,6 +1501,9 @@ exchange_account_connect (ExchangeAccount *account)
 			account->default_timezone = g_strdup (timezone);
 	}
 
+	if (!setup_account_hierarchies (account)) 
+		return NULL;
+
 	/* Find the password expiery peripod and display warning */
 	find_passwd_exp_period(account, entry);
 	
@@ -1327,113 +1515,24 @@ exchange_account_connect (ExchangeAccount *account)
 					    E2K_GLOBAL_CATALOG_LOOKUP_QUOTA,
                                             &entry);	
 	e2k_operation_free (&gcop);
-	
+
+	/* FIXME: quota warnings are not yet marked for translation!! */
+	/* FIXME: warning message should have quota limit value and optionally current
+	 * usage 
+	 */
 	if (gcstatus == E2K_GLOBAL_CATALOG_OK) {
 
-		if (entry->quota_norecv) {
+		if (entry->quota_norecv && account->mbox_size >= entry->quota_norecv) {
 			quota_msg = g_strdup_printf ("You have exceeded your quota for storing mails on this server. Your current usage is : %d . You will not be able to either send or recieve mails now\n", entry->quota_norecv);
-		} else if (entry->quota_nosend) {
+		} else if (entry->quota_nosend && account->mbox_size >= entry->quota_nosend) {
 			quota_msg = g_strdup_printf ("You are nearing your quota available for storing mails on this server. Your current usage is : %d . You will not be able to send mails till you clear up some space by deleting some mails.\n", entry->quota_nosend);
-		} else if (entry->quota_warn) {
+		} else if (entry->quota_warn && account->mbox_size >= entry->quota_warn) {
 			quota_msg = g_strdup_printf ("You are nearing your quota available for storing mails on this server. Your current usage is : %d . Try to clear up some space by deleting some mails.\n", entry->quota_warn);
 		}
 		
 		if (quota_msg)
 			e_notice (NULL, GTK_MESSAGE_INFO, quota_msg);
 	}
-
-	/* Set up Personal Folders hierarchy */
-	phys_uri_prefix = g_strdup_printf ("exchange://%s/personal",
-					   account->priv->uri_authority);
-	hier = exchange_hierarchy_webdav_new (account,
-					      EXCHANGE_HIERARCHY_PERSONAL,
-					      _("Personal Folders"),
-					      phys_uri_prefix,
-					      account->home_uri,
-					      account->priv->identity_name,
-					      account->priv->identity_email,
-					      account->priv->source_uri,
-					      TRUE);
-	setup_hierarchy (account, hier);
-	g_free (phys_uri_prefix);
-	personal_hier = hier;
-
-	/* Favorite Public Folders */
-	phys_uri_prefix = g_strdup_printf ("exchange://%s/favorites",
-					   account->priv->uri_authority);
-	hier = exchange_hierarchy_favorites_new (account,
-						 _("Favorite Public Folders"),
-						 phys_uri_prefix,
-						 account->home_uri,
-						 account->public_uri,
-						 account->priv->identity_name,
-						 account->priv->identity_email,
-						 account->priv->source_uri);
-	setup_hierarchy (account, hier);
-	g_free (phys_uri_prefix);
-	account->priv->favorites_hierarchy = hier;
-
-	/* Public Folders */
-	phys_uri_prefix = g_strdup_printf ("exchange://%s/public",
-					   account->priv->uri_authority);
-	hier = exchange_hierarchy_webdav_new (account,
-					      EXCHANGE_HIERARCHY_PUBLIC,
-					      /* i18n: Outlookism */
-					      _("All Public Folders"),
-					      phys_uri_prefix,
-					      account->public_uri,
-					      account->priv->identity_name,
-					      account->priv->identity_email,
-					      account->priv->source_uri,
-					      FALSE);
-	setup_hierarchy (account, hier);
-	g_free (phys_uri_prefix);
-
-	/* Global Address List */
-	phys_uri_prefix = g_strdup_printf ("gal://%s/gal",
-					   account->priv->uri_authority);
-						     /* i18n: Outlookism */
-	hier = exchange_hierarchy_gal_new (account, _("Global Address List"),
-					   phys_uri_prefix);
-	setup_hierarchy (account, hier);
-	g_free (phys_uri_prefix);
-
-	/* Other users' folders */
-	d = opendir (account->storage_dir);
-	if (d) {
-		while ((dent = readdir (d))) {
-			if (!strchr (dent->d_name, '@'))
-				continue;
-			dir = g_strdup_printf ("%s/%s", account->storage_dir,
-					       dent->d_name);
-			hier = exchange_hierarchy_foreign_new_from_dir (account, dir);
-			g_free (dir);
-			if (!hier)
-				continue;
-
-			setup_hierarchy_foreign (account, hier);
-		}
-		closedir (d);
-	}
-
-	/* Scan the personal and favorite folders so we can resolve references
-	 * to the Calendar, Contacts, etc even if the tree isn't
-	 * opened.
-	 */
-	fresult = exchange_hierarchy_scan_subtree (personal_hier,
-						   personal_hier->toplevel);
-	if (fresult != EXCHANGE_ACCOUNT_FOLDER_OK) {
-		account->priv->connecting = FALSE;
-		return NULL;
-	}
-
-	fresult = exchange_hierarchy_scan_subtree (
-		account->priv->favorites_hierarchy,
-		account->priv->favorites_hierarchy->toplevel);
-	if (fresult != EXCHANGE_ACCOUNT_FOLDER_OK) {
-		account->priv->connecting = FALSE;
-		return NULL;
-	}
 	
 	account->priv->connected = TRUE;
 	account->priv->account_online = TRUE;
@@ -1446,6 +1545,25 @@ exchange_account_connect (ExchangeAccount *account)
 	return account->priv->ctx;
 }
 
+/**
+ * exchange_account_is_offline_sync_set:
+ * @account: an #ExchangeAccount
+ *
+ * Return value: TRUE if offline_sync is set for @account and FALSE if not.
+ */
+void
+exchange_account_is_offline_sync_set (ExchangeAccount *account, int *mode)
+{
+	*mode = UNSUPPORTED_MODE;
+
+	g_return_if_fail (EXCHANGE_IS_ACCOUNT (account));
+
+	if (account->priv->offline_sync)
+		*mode = OFFLINE_MODE;
+	else
+		*mode = ONLINE_MODE;
+}
+
 /**
  * exchange_account_get_context:
  * @account: an #ExchangeAccount
@@ -1670,7 +1788,8 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 {
 	ExchangeAccount *account;
 	char *enc_user, *mailbox;
-	const char *param, *proto, *owa_path, *pf_server;
+	const char *param, *proto="http", *owa_path, *pf_server, *owa_url; 
+	const char *passwd_exp_warn_period, *offline_sync;
 	E2kUri *uri;
 
 	uri = e2k_uri_new (adata->source->url);
@@ -1728,6 +1847,18 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 		if (param)
 			account->priv->ad_limit = atoi (param);
 	}
+	
+	passwd_exp_warn_period = e2k_uri_get_param (uri, "passwd_exp_warn_period");
+	if (!passwd_exp_warn_period || !*passwd_exp_warn_period)
+		account->priv->passwd_exp_warn_period = -1;
+	else
+		account->priv->passwd_exp_warn_period = atoi (passwd_exp_warn_period);
+
+	offline_sync = e2k_uri_get_param (uri, "offline_sync");
+	if (!offline_sync) 
+		account->priv->offline_sync = FALSE;
+	else 
+		account->priv->offline_sync = TRUE;
 
 	owa_path = e2k_uri_get_param (uri, "owa_path");
 	if (!owa_path || !*owa_path)
@@ -1739,7 +1870,19 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 	if (!pf_server || !*pf_server)
 		pf_server = uri->host;
 
-	proto = e2k_uri_get_param (uri, "use_ssl") ? "https" : "http";
+	/* We set protocol reading owa_url, instead of having use_ssl parameter 
+	 * because we don't have SSL section anymore in the account creation
+	 * druid and account editor
+	 */
+	/* proto = e2k_uri_get_param (uri, "use_ssl") ? "https" : "http"; */
+
+	owa_url = e2k_uri_get_param (uri, "owa_url");
+	if (owa_url) {
+		account->priv->owa_url = g_strdup (owa_url); 
+		if (!strncmp (owa_url, "https:", 6))
+			proto = "https";
+	}
+
 	if (uri->port != 0) {
 		account->priv->http_uri_schema =
 			g_strdup_printf ("%s://%%s:%d/%s/%%s/",
